<?php
/**
* File class
* 
* Represents an edonkey file (upload or download)
*
* @author   Don Seiler <don@NOSPAM.seiler.us>
* @author Moritz Warning <mwarning@users.sourceforge.net>
*/
class MLD_Download {

	var $isvalid = FALSE;
/**
* MD4 Hash
*/
	var $hash;

/**
* Network (eDonkey, Direct Connect, Open Napster, BitTorrent, etc.)
*/
	var $network;

/**
* mldonkey Number
*/
	var $nr;

/**
* Transfer rate
*/
	var $speed;

/**
* Size of file in byte
*/
	var $size;

/**
* Amount downloaded in byte
*/
	var $downloaded;


/**
* Percent downloaded
*/
	var $percent;


/**
* Estimated time of completion at current $speed
*/
	var $eta;

/**
* Name of file
*/
	var $name;

/**
* Downloads availability in the Network
*/
	var $availability;


/**
* Days in download list
*/
	var $daysinlist;

/**
* Time since last seen
*/
	var $lastseen;
	
/**
* Whether or not file is finished downloading
*/
	var $finished = FALSE;

/**
* File aging information
* (conglomeration of daysinlist and lastseen)
*/
	var $old;

/**
* Priority
*
*/
	var $priority;

/**
* Array of different names for this file
*/
	var $names;
	
/**
*  String of source count
*/
	var $numsources;
	
/**
* Chunks
*/
	var $chunks;
	
/**
* Array of sources for this file
*/
	var $sources;

	      
	function MLD_Download($line = NULL, $finished = FALSE)
	{
		if(!isset($line) OR ($line[0] != '[') OR preg_match("/^\s*\[ Num/", $line))
		{
			return;
		}
		$this->finished = $finished;
		
		if($this->finished) { // Parse finished Downloads
			preg_match("/^\[([a-zA-Z]+)\s*(\d+)\s*\]\s+(.+)\s+(\d+)\s+([0-9A-F]{32})/", $line, $found);
			
			$this->network = $found[1];
			$this->nr = (int) $found[2];
			$this->name = trim($found[3]);
			$this->size = (float) $found[4];
			$this->downloaded = (float) $this->size;
			$this->hash = $found[5];
			$this->percent = 100;
			$this->eta = 0;
			$this->speed = "finished";
			$this->old = "-/-";
			$this->numsources = "-/-";
			$this->priority = "-";
			$this->isvalid = TRUE;
			
		} else { // Parse Downloads
			preg_match("/^\[([a-zA-Z])\s*(\d+)\]\s+(.*)\s+(\d+\.\d)\s+(\d+\.\d\w*|\d\w*)\s+(\d+.\d\w*|\d\w*)\s+([\d:]+|\d\w*|-)\s+(\d+):(\d+|-)\s+(\d+\/\d+)\s+(\d+\.\d|-|Paused|Finished|Queued)\s+(\-?\d+)/",$line,$found); 
		
			$this->network = $found[1];
			switch($this->network) {
				case "D": $this->network = "Donkey"; break;
				case "B": $this->network = "BitTorrent"; break;
				case "T": $this->network = "FileTP"; break;
				case "F": $this->network = "FastTrack"; break;
				case "G": $this->network = "Gnutella2"; break;
			}
			
			$this->nr = $found[2];
			$this->name = trim($found[3]);
			$this->percent = (float) $found[4];
			$this->downloaded = (float) byteSize($found[5]);
			$this->size = (float) byteSize($found[6]);
			$this->lastseen = $found[7];
			$this->old = $found[8]."/". $found[9];
			$this->daysinlist = $found[8];
			$this->numsources = $found[10];
			$this->speed = $found[11];
			$this->priority = $found[12];
			
			$this->eta = eta($this->size - $this->downloaded, $this->speed);
			
			$this->isvalid = TRUE;
		}
	}
	
	function getDetails($short = FALSE) //$short is used to prevent parsing of the whole telnet output when only the hash is needed
	{
		if(!isset($this->nr)) { return; }
		$mldarray = $_SESSION['interface']->mldonkey("vd ".$this->nr);
		$mode = 1;
		
		if(strlen($mldarray[0]) < 26) {  //coarse test for version output >2.5-30-4 or >2.5-16v-3
			//parse vd <num> output for MLD up from 2.5..30-5
			foreach($mldarray as $line) {

			switch($mode) {
			
			case 1:
				if(preg_match('/^\[(\w+)\s+(\d+)\]/', $line, $found)) {
					$this->network = $found[1];
					$this->nr = (float) $found[2];
					$mode = 2;
				} break;
			case 2:	
				$this->name = trim($line);
				$mode = 3;
			case 3:
				if(preg_match('/^(urn:[0-9a-z]*:)?([0-9A-Z]{32})/i', $line, $found)) {
					$this->hash = $found[2];
					$mode = 4;
				} break;
			case 4:
				if(preg_match('/^Total\s+(\d+)/', $line, $found)) {
					$this->size = (float) $found[1];
					$mode = 5;
				} break;
			case 5:
				if(preg_match('/^Partial\s+(\d+)/', $line, $found)) {
					$this->downloaded = (float) $found[1];
					$mode = 6;
				} break;
			case 6:
				if(preg_match('/^priority\s+(\-?\d+)/i', $line, $found)) {
					$this->priority = (int) $found[1];
					$mode = 7;
				} break;
			case 7:
				if(preg_match('/^Chunks:\s+\[(\d*)/', $line, $found)) {
					$this->chunks = $found[1];
					$this->availability = availability($this->chunks);
					$mode = 8;
				} break;
			case 8:
				if(preg_match('/^Probable name:\s+(.*)/', $line, $found)) {
					//skip redundant name
					$this->name = $found[1];
				}
				
				if(str_pad("", strlen($this->hash), "0") == $this->hash) { //when hash consist of 0
					if($this->size) {
						$this->hash = $this->size;
					} else {
						$this->hash = $found[1];
					}
				}
				if($short) { //stop parsing here, we want the hash only
					$this->isvalid = TRUE;
					return;
				}
				
				$mode = 9;
				break;
			
			case 9:	if(preg_match('/\s*\((.*)\)/', $line, $found)) {
					$this->names[] = $found[1];
					break;
				} else {
					$mode = 10;
				}
			case 10: if(preg_match('/^(\d+)\s+sources:/i', $line, $found)) {
					$this->numsources = (int) $found[1];
					$mode = 11;
				} break;
			case 11: if(preg_match('/\s*\[\s+\d+\]\s+(.*)/', $line, $found)) {
					$this->sources[] = $found[1];
					$this->isvalid = TRUE;
				} break;
			}
			}
		} else {
			//parse vd <num> output for MLD up to 2.5.30-4 and all 2.5-16
			foreach($mldarray as $line) {
			
				if($mode == 1 AND preg_match("/^\[(\w+)\s+(\d+)\]\s+(.*)\s+(\d+)\s+(\d+)\s*/", $line, $found)) {
				
					$this->network = $found[1];
					$this->nr = (int) $found[2];
					$this->size = (float) $found[4];
					
					//use real hash when available else ...
					//it is important that the hash is a good identifier for the file
					//the "hash" is used when mldonkey restarts for database update of file ids
					if(preg_match("/^[0-9A-Fa-f]{32}$/", trim($found[3]))) {
						$this->hash = (string) trim($found[3]); //hash value exist
					} elseif($this->size != 0) {
						$this->hash = (string) $this->size; //bad :(
					} else {
						$this->hash = (string) trim($found[3]); //really bad :(
					}
					
					//$this->hash = (string) $this->size; //bad hack :(
					
					$this->downloaded = (float) $found[5];
					
					if($short) { //skip rest
						$this->isvalid = TRUE;
						return;
					}
					$mode = 2;
					
				} elseif($mode == 2 AND preg_match("/^priority\s+(\d+)\s*/", $line, $found)) {
				
					$this->priority = (int) $found[1];
					$mode = 3;
					
				} elseif($mode == 3 AND preg_match("/^Chunks:\s+\[(\d+)\]\s*/", $line, $found)) {
				
					$this->chunks = $found[1];
					$this->availability = availability($this->chunks);
					$mode = 4;
					continue;

				}
				
				if($mode == 4) {
					if( preg_match("/\s*\((.+)\)/", $line, $found)) {
						$this->names[] = $found[1];
					} else { $mode = 5; }
					
				}
				if($mode == 5) {
					if(preg_match("/\s*(\d+)\s+sources:/", $line, $found)) {
						if(!isset($this->numsources)) {
							$this->numsources = (int) $found[1]; //this info is not as detailed as via <vd>
						}
					} else { $mode = 6; }
					
				}
				if($mode == 6 AND preg_match("/\s*(\[.+)\s*/", $line, $found)) {
				
					$this->sources[] = $found[1];
					$this->isvalid = TRUE;
				}
			}
		}
	}
	
	function setFilename($filename)
	{
		$this->name = $filename;
	}
	
}
