<?php
require_once("./client/mldonkey/MLD_Download.php");
require_once("./client/mldonkey/MLD_Server.php");
require_once("./client/mldonkey/MLD_Search.php");
require_once("./client/mldonkey/MLD_SearchResult.php");
require_once("./client/mldonkey/MLD_Client.php");
require_once("./client/mldonkey/MLD_Shared.php");
require_once("./client/mldonkey/MLD_BandwidthStat.php");
require_once("./client/mldonkey/MLD_Upload.php");
require_once("./client/mldonkey/MLD_ServerConnection.php");
require_once("./client/mldonkey/MLD_Upstats.php");
require_once("./client/Shared.php");

class MLD_Interface extends Cache {
	
	var $upload_rate = 0;
	var $download_rate = 0;

	//enable extra output for multiuser purpose
	//such as dowload numbers for started downloads, searches etc.
	var $multiuser = TRUE;
	
	//connection timeout
	var $timeout = 20;
	
	//connection settings
	var $host;
	var $port;
	var $user;
	var $passwd;
	
	function MLD_Interface($host = "localhost", $port = 4000, $user = NULL, $passwd = NULL) {
		
		//set connection settings
		$this->host = $host;
		$this->port = (int) $port;
		$this->user = $user;
		$this->passwd = $passwd;
		
		//init cache
		$this->cache_add("nr_name", "cache_set_names");
		$this->cache_add("nr_size", "cache_set_hash_size");
		$this->cache_add("nr_hash", "cache_set_hash_size");
	}

	function postInit ()
	{
		$this->mldonkey("set max_name_len 200");
	}

	/**
	* Makes test connection to see if mldonkey client is running
	*
	* @author   Don Seiler <don@NOSPAM.seiler.us>
	*/
	function alive()
	{      
		if($open = @pfsockopen($this->host, $this->port, $errno, $errstr, $this->timeout)) {
			fclose($open);
			return TRUE;
		} else {
			return FALSE;
		}
	}
	
	function kill() {
		$this->mldonkey("kill\n");
		return TRUE;
	}
	
	function check_MLDrestart() {
		//data for reload protection, to detect corrupt database when the MLD restarts
		$MLDcount = 0;
		$MLDmax = 0;
		$MLDid1 = NULL;
		$MLDid2 = NULL;
		$nrs = $this->get_all_nrs();
		if(!empty($nrs)) {
			$MLDcount = count($nrs);
			$MLDmax = max($nrs);
			$MLDid1 = $nrs[0]; //get first nr
			$MLDid2 = $nrs[$MLDcount - 1]; //get last nr
		}
		
		//debug output
		//echo "new MLDid1: |".$MLDid1."|<br>";
		//echo "old MLDid1: |".$_SESSION['MLDid1']."|<br>";
		//echo "new MLDid2: |".$MLDid2."|<br>";
		//echo "old MLDid2: |".$_SESSION['MLDid2']."|<br>";
		//echo "old MLDid2: |".$MLDcount."|<br>";
		
		if (!empty($nrs) AND (($MLDmax - $MLDcount) == 0)) {//if ($MLDmax-$MLDcount)==0 it is hard to decide if the mldonkey was restarted
			if($_SESSION['MLDid1']!= $MLDid1 OR $_SESSION['MLDid2']!= $MLDid2 OR !isset($_SESSION['MLDid1'])) {
				$_SESSION['MLDid1'] = $MLDid1;
				$_SESSION['MLDid2'] = $MLDid2;
				return TRUE; //after this point, verify_sb should executed
			}
		}
		return FALSE;
	}
	
	function get_name($nrs) {
		return $this->cache_get("nr_name", $nrs);
	}
	
	function get_size($nrs) {
		return $this->cache_get("nr_size", $nrs);
	}
	
	function get_hash($nrs) {
		return $this->cache_get("nr_hash", $nrs);
	}
	
	function cache_set_names($nr) {
		$mldarray = $this->mldonkey("vd");
		$temp = array();
		foreach($mldarray as $line) {
			$file = new MLD_Download($line, FALSE);
			if($file->isvalid) {
				$this->cache_set("nr_name", $file->nr, $file->name);
				if($file->nr == $nr) {
					return TRUE;
				}
			}
		}
		return FALSE;
	}
	
	function cache_set_hash_size($nr, $retry = 5) {
		
		//if nr and hash is not in cache
		//$mldarray = $this->mldonkey("set print_all_sources false", "vd ".$nr , "set print_all_sources true");
	
		$file = new MLD_Download();
		$file->nr = $nr;

		$file->getDetails(TRUE);
		
		if($file->isvalid) {
			return $this->cache_set("nr_hash", $file->nr, $file->hash) AND $this->cache_set("nr_size", $file->nr, $file->size);
		} elseif($retry >= 0) {
			sleep(1);
			return $this->cache_set_hash_size($file->nr, $retry - 1); //retry to get size when it is not yet set by mldonkey
		} else {
			return NULL;
		}
	}
	
/*
* getAllNrs parse all download numbers out of the MLDonkey an return them as an array
*
* @author Moritz Warning <mwarning@users.sourceforge.net>
*/
	function get_all_nrs() {
		$mldarray = $this->mldonkey("vd");
		$temp = array();
		foreach($mldarray as $line) {
			$file = new MLD_Download($line, FALSE);
//echo "<pre>";print_r($file); echo "</pre>"; //debug output
			if($file->isvalid) {
				$temp[] = $file->nr;
			}
		}
		return $temp;
	}
	
	function get_nr($hashes) {
		
		if(is_array($hashes)) {

			$result = array();
			$tmp = array();
			$c = count($hashes);
			$i = 0;
			$nrs = $this->get_all_nrs();
			foreach($nrs as $nr) {
				$hash = $this->get_hash($nr);
				if(in_array($hash, $hashes)) {
					$tmp[$hash] = $nr;
					$i++;
					if($i == $c){ break; }
				}
			}
			foreach($hashes as $hash) {
				$result[$hash] = $tmp[$hash];
			}
			return $result;
		} else {
			$temp = $this->get_all_nrs();
			rsort($temp); //the new downloads have a high number
			foreach($temp as $nr) {
				$hash1 = $this->get_hash($nr);
				if($hashes == $hash1) {
					return $nr;
				}
			}
			return FALSE;
		}
	}
	
//serach*
	function addSearch($search) {
		
		//handle german umlauts and other substitutions
		$trans = array( "" => "ae", "" => "oe", "" => "ue", "" => "ss", "-" => " ");
		$query = str_replace(array_keys($trans), array_values($trans), $search['keywords']);
		
		$query = trim($query);
		
		if(empty($query)) { return FALSE; }
		
		//$query = '"'.$query.'"'; //need to check if every version allow qoutes
		
		if(isset($search['minsize'])) {
			$query .= " -minsize ".$search['minsize']*1048576;
		}
		
		if(isset($search['maxsize'])) {
			$query .= " -maxsize ".$search['maxsize']*1048576;
		}
		
		if(isset($search['network'])) {
			$query .= " -network ".$search['network'];
		}
		
		$mldarray = $this->mldonkey("s ".$query);
		$tmp = explode(" ", $mldarray[0]);
		if($this->multiuser) {
			return (int) $tmp[1];
		} else {
			return (bool) $tmp[1];
		}
	}

//function to validate search items
	function is_valid_search($search) {
		
	}

	function delSearches($searchnums) {
		if(is_array($searchnums)){
			$this->mldonkey("forget ".implode(" ", $searchnums));
		} else {
			$this->mldonkey("forget ".$searchnums);
		}
		return TRUE;
	}
	
	function getAllSearches() {
		$searches = array();
		$mldarray = $this->mldonkey("vs");
		foreach($mldarray as $line) {
			$search = new MLD_Search($line);
			if($search->isvalid) {
				$searches[] = $search;
			}
		}
		return $searches;
	}

	function getSearchResults($searchnum) {
		$results = array();
		$mldarray = $this->mldonkey("vr " . $searchnum);
		foreach($mldarray as $line) {
			$result = new MLD_SearchResult($line);
			if($result->isvalid) {
				$results[] = $result;
			}
		}
		return $results;
	}

	function downloadSearch($sid, $srns) {
		//view search results because files
		//from last shown search are downloaded :/
		$this->mldonkey("vr " . $sid);
		
		if($this->multiuser) {
			$nrs = array();
			foreach((array) $srns as $srn) {
				$max_nr = $this->getMaxNr();
				$mldarray = $this->mldonkey("d ".$srn);
				$done = FALSE;
				foreach($mldarray as $line) {
					if(preg_match("/^download started/i", $line)) {
						$done = TRUE;
					}
				}
				if(!$done) { continue; }
			
				if($nr = $this->getHigherNr($max_nr, 3)) {
					$nrs[] = $nr;
				}
			}
			
			return $nrs;
		} else {
			foreach((array) $srns as $srn) {
				$this->mldonkey("d ".$srn);
			}
			return TRUE;
		}
	}
	
	function forceDownload() {
		$this->mldonkey("force_download");
		return TRUE;
	}
	
	function commitDownloads() {
		$mldarray = $this->mldonkey("commit");
		foreach ($mldarray as $line) {
			if(preg_match("/^Commited/", $line)) {
				return TRUE;
			}
		}
		return FALSE;
	}

//Option*
	function getOptions($all) {
		if($all) { $op = "voo"; }
		else { $op = "vo"; }
		$mldarray = $this->mldonkey($op);
		
		$options = array();
		foreach($mldarray as $line) {
			if(preg_match("/(.+) = (.*)\s*/",$line,$found)) {
				$options[$found[1]] = $found[2];
			}
		}
		return $options;
	}
	
	function getOption($key) {
		$options = $this->getOptions(TRUE);
		if(isset($options[$key])) {
			return $options[$key];
		} else {
			return NULL;
		}
	}
	
	function saveOptions() {
		$mldarray = $this->mldonkey("save");
		foreach($mldarray as $line)
			if(preg_match("/^saved/", $line)) {
				return TRUE;
			}
		return FALSE;
	}
//share*
	function share($priority, $dir) {
		if(!is_numeric($priority)) { return FALSE; }
		
		$mldarray = $this->mldonkey("share " . $priority . " " . $dir);
		foreach($mldarray as $line) {
			if(preg_match("/^directory already shared/",$line)) {
				$_SESSION["message"] .= '<div class="status">' . _("Directory Already Shared!") . "</div>\n";
				return FALSE;
			}
			if(preg_match("/^directory added/",$line)) {
				return TRUE;
			}
		}
		return FALSE;
	}

	function reshare() {
	
		$mldarray = $this->mldonkey("reshare");
		foreach($mldarray as $line) {
			if(preg_match("/^check done/",$line)) {
				return TRUE;
			}
		}
		return FALSE;
	}

	function unshare($directory) {
	
		$mldarray = $this->mldonkey("unshare " . trim($directory));
		foreach($mldarray as $line) {
			if(preg_match("/^directory already unshared/",$line)) {
				$_SESSION["message"] .= '<div class="status">' . _("Directory Already Unshared!") . "</div>\n";
				return FALSE;
			}
			if(preg_match("/^directory removed/",$line)) {
				return TRUE;
			}
		}
		return FALSE;
	}
	
	function getShares() {
		$mldarray = $this->mldonkey("shares");
		$dirs = array();
		foreach ($mldarray as $line) {
			$dir = new MLD_Shared($line);
			if ($dir->isvalid) {
				$dirs[] = $dir;
			}
		}
		return $dirs;
	}
	
	function getUpstats() {
		$mldarray = $this->mldonkey("upstats");
		$files = array();
		foreach ($mldarray as $line) {
			$file = new MLD_Upstats($line, TRUE);
			if ($file->isvalid) {
				$files[] = $file;
			}
		}
		//[bug] mldonkey give multiple entries for the same file => need to unify
		$files = unique_objects($files, "name", TRUE);
		foreach($files as $index => $object) {
			if(is_array($object)) {
				$files[$index] = merge_objects($object, array("name"), FALSE); //merge all object vars except  var "name"
			}
		}
		
		return $files;
	}	
//Upload*
	function setRate() {
		$mldarray = $this->mldonkey("bw_stats");
		
		$stats = new MLD_BandwidthStat($mldarray[0]);
		if ($stats->isvalid) {
			$this->download_rate = $stats->downrate;
			$this->upload_rate = $stats->uprate;
		}
	}
	
	function getUploaders() {
		$mldarray = $this->mldonkey("uploaders");
		$next = FALSE;
		$ufiles = array();
		foreach($mldarray as $line) {
			if($next) {
				if(preg_match("/^\s*filename/",$line)) {
					$file->setFilename(substr($line, 10));
				} else { $file->setFilename("???"); }
				$ufiles[] = $file;
				$next = FALSE;
			} else {
				$file = new MLD_Upload($line, TRUE);
				if($file->isvalid) { $next = TRUE; }
			}
		}
		return $ufiles;
	}
	
//Server*
	/*
	* get server stats to check high/low id connection
	*/
	function getServerStats() {
		$mldarray = $this->mldonkey("id");
		foreach($mldarray as $line) {
			$server_stat = new MLD_ServerConnection($line);
			if(!$server_stat->isvalid){ continue; }
			$server_stats[$server_stat->serverIp] = $server_stat->statusId;
		}
		return $server_stats;
	}
	
	function getServers($all) {
		//every two lines describe one server, so this is a bit tricky	
		if($all) { $mldarray = $this->mldonkey("vma"); }
		else { $mldarray = $this->mldonkey("vm"); }
		$c = 0;
		$firstline = "";
		$server_stats = $this->getServerStats();
		
		//concatenate two lines
		foreach($mldarray as $line) {
			$c++;
			if($c%2 == 0) {
				$server = new MLD_Server($firstline.$line);
				if(!$server->isvalid) {
					$firstline = $line;
					$c = 1;
					continue;
				}
			} else {
				$firstline = $line;
				continue;
			}
			//set server Id (high/low)
			if($server->status == "Connected") {
				$id = $server_stats[$server->ip]; //get server id (high or low)
				$server->setId($id);
			} else {
				$server->setId("&nbsp;");
			}
			$servers[] = $server;
		}
		return $servers;
	}

	function connectServers($serverNrs) {
		$commandarray = array();
		foreach((array) $serverNrs as $nr) {
			$commandarray[] = "c $nr";
		}
		$this->mldonkey($commandarray);
		return TRUE;  //need to check result
	}
	
	function disconnectServers($serverNrs) {
		$commandarray = array();
		foreach((array) $serverNrs as $nr) {
			$commandarray[] = "x $nr";
		}
		$this->mldonkey($commandarray);
		return TRUE;   //need to check result
	}
	
	function removeServers($serverNrs) {
		$command = "rem";
		foreach((array) $serverNrs as $nr) {
			$command .= " $nr";
		}
		$this->mldonkey($command);
		return TRUE;     //need to check result
	}

/**
* Grabs version of MLDONKEY
*
* @author   Don Seiler <don@NOSPAM.seiler.us>
*/	
    	function getClientVersion()
	{
		if($this->alive()) {
			$mldarray = $this->mldonkey("version");
			
			if(preg_match('/^MLNet\s+[0-9\.\-\+\w]*/', $mldarray[0],$result)) {
				return substr($result[0],6);
			}
		}
		return "???";
	}
	
	function getAllClients() {
		$mldarray = $this->mldonkey("vc all");
		foreach($mldarray as $line) {
			$client = new MLD_Client($line);
			if($client->isvalid) {
				$clients[] = $client;
			}
		}
		return $clients;
	}
	
	function verifyChunks($nr) {
		$mldarray = $this->mldonkey("verify_chunks ".$nr);
	}
	
//Friends*

	function remFriend($identifier) {
		$mldarray = $this->mldonkey("friend_remove ".trim($identifier));
		return TRUE;
	}
	
	function addFriend($ip, $port = NULL) {
		if($port == NULL) {
			$mldarray = $this->mldonkey("friend_add ".trim($ip)); //ip is an identifier in this case, no real IP
			return TRUE;
		}
		
		$mldarray = $this->mldonkey("afr ".trim($ip)." ".trim($port));
		foreach($mldarray as $line) {
			if(trim($line) == "MLdonkey command-line:") {
				return FALSE;
			} else {
				if (trim($line) == "friend added") {
					return TRUE;
				}
			}
		}
		return FALSE;
	}

	function downloadLink($link, $type) {
	
		if($this->multiuser) {
			switch($type) {
				case "Donkey":	return $this->download_ed2k_link($link); break;
				case "HTTP":
				case "BitTorrent":
				case "???":	return $this->download_link($type, $link); break;
				default:	return FALSE;
			}
		} else {
			if($type == "HTTP") {
				$mldarray = $this->mldonkey('http "'.$link.'"');
			} else {
				$mldarray = $this->mldonkey('dllink "'.$link.'"');
			}
		
			foreach($mldarray as $line) {
				if(preg_match("/done|started|parsing/i", $line)) { return TRUE; }
			}
			return FALSE;
		}
	}

	function download_link($type, $link) {

		$mldarray = NULL;
//echo "$type: $link".BN; flush(); //debug output
		$prev_nr = $this->getMaxNr();
//echo "prev_nr:$prev_nr".BN; flush(); //debug output
		if($type == "HTTP") {
			$mldarray = $this->mldonkey('http "'.$link.'"');
		} else {
			$mldarray = $this->mldonkey('dllink "'.$link.'"');
		}
		
		$done = FALSE;
		foreach($mldarray as $line) {
			if(preg_match("/done|started|parsing|added/i", $line)) {
				$done = TRUE;
			}
		}
		if(!$done) { return FALSE; }

		//wait some time until torrent file was downloaded and BT starts
		if($type == "BitTorrent") {
			sleep(10);
		}
		
		//wait for new download appearing
		if($nr = $this->getHigherNr($prev_nr, 7)) {
			return $nr;
		}
		
		//extract filename from link
		$name = basename($link);
		if(strtolower(substr($name, -8)) == '.torrent') {
			$name = substr($name, 0, -8);
		}
		
		//try to find already downloading file by name //this might be a minor privacy flaw...
		if($nr = $this->get_nr_by_name($name)) {
			return $nr;
		}
		
		return FALSE;
	}
	
	function download_ed2k_link($link) {
		if(preg_match_all('/ed2k:\/\/\|file\|(.+)\|(\d+)\|([a-f0-9]{32})\|/i',$link, $result)) {  //get MD4-hash
			
			$name = $result[1][0];
			$size = $result[2][0];
			$hash = strtoupper($result[3][0]);
		
			$mldarray = $this->mldonkey('dllink "'.$link.'"');
			
			//MLdonkey telnet doesn't match the link when the file is already downloaded, this doesn't work for bittorent, because of no hash  :(
			if($nr = $this->get_nr($hash)) {
				return $nr;
			}
			
			$done = FALSE;
			foreach($mldarray as $line) {
				if(preg_match("/done|parsing/i", $line)) {
					$done = TRUE;
				}
			}
			if(!$done) { return FALSE; }
			
			//try to find new file by hash
			for($i = 0; $i < 5; $i++) {
				if($nr = $this->get_nr($hash)) {
					return $nr;
				}
				sleep(1);
			}
			
			//try to find already downloading file by name //this might be a minor privacy flaw...
			if($nr = $this->get_nr_by_name($name)) {
				return $nr;
			}
		}
		return FALSE;
	}
	
	//find already quequed file by name
	function get_nr_by_name($names) {
		$nrs = $this->get_all_nrs();
		
		$count = count($names);
		$i = 0;
		
		$ret = array();
		foreach($nrs as $nr) {
			$tmp = $this->get_name($nr);
			if(!in_array($tmp, (array) $names)) {
				continue;
			}
			$ret[$tmp] = $nr;
			if(++$i == $count) {
				if(is_string($names)) {
					return $nr;
				}
				break;
			}
		}
		return $ret;
	}
	
	function getMaxNr() {
		$all_nrs = $this->get_all_nrs();
		array_push($all_nrs, 0);
		return max($all_nrs);
	}
	
	function getHigherNr($old_max_nr, $wait) {
	
		for($i = 0; $i < $wait; $i++) {
			$new_max_nr = $this->getMaxNr();
//echo "new_max_nr:$new_max_nr<br>\n"; flush(); //debug output
			if($new_max_nr > $old_max_nr) {
				return $new_max_nr;
			}
			sleep(1);
		}
		return FALSE;
	}
//Downloads*

	function cancelDownloads($nrs) {
		$commands = array();
		foreach((array) $nrs as $nr) {
			$commands[] = 'cancel '.$nr;
			$commands[] = "confirm yes";
		}
		$this->mldonkey($commands);
		return TRUE;
	}
	
	function resumeDownloads($nrs) {
		$commands = array();
		foreach((array) $nrs as $nr) {
			$commands[] = 'resume '.$nr;
		}
		$this->mldonkey($commands);
		return TRUE;
	}
	
	function pauseDownloads($nrs) {
		$commands = array();
		foreach((array) $nrs as $nr) {
			$commands[] = 'pause '.$nr;
		}
		$this->mldonkey($commands);
		return TRUE;
	}
	
	function priorDownloads($nrs, $prio) {
		$command = "priority =".$prio;
		$command .= " ".implode((array) $nrs, " ");
		$this->mldonkey($command);
		return TRUE;
	}
	
	function renameDownload($nr, $newname) {
		$this->mldonkey("rename ".$nr.' "'.$newname .'"');
		$this->cache_reset("nr_name", $nr); //clear cache in name
		return TRUE;
	}
	
	function getAllDownloads() {
		$mldarray = $this->mldonkey("vd");
		$files = array();
		$finished = FALSE;
		
		$stats = new MLD_BandwidthStat($mldarray[0]);
		if ($stats->isvalid) {
			$this->download_rate = $stats->downrate;
			$this->upload_rate = $stats->uprate;
		}
		
		foreach($mldarray as $line) {
			$file = NULL;
			if (preg_match("/^Downloaded\s+\d+\s+files/", $line)) {
				//test if download is finished, but still in download lst
				$finished = TRUE;
			}
			$file = new MLD_Download($line, $finished); //some lines does not content file information so some $file object are NULL
			
			if($file->isvalid) {
				$files[] = $file;
			}
		}
		return $files;
	}
	
	function setOptions($options) {
		$commandarray = array();
		foreach($options as $key => $value) {
			if (strpos($value, " ") === FALSE) {
				$commandarray[] = "set $key $value";
			} else {
				$commandarray[] = "set $key \"$value\"";
			}
		}
		if (count($commandarray) > 0) {
			$this->mldonkey($commandarray);
		}
		return TRUE; //need to check success
	}
	
	function getDownloadDetails($nr) {
		$mldarray = $this->mldonkey("set max_name_len 200", "vd");
		foreach($mldarray[1] as $line) {
			$file = new MLD_Download($line);
			if($file->nr == $nr) { break; }
		}
		if($file == NULL) { return NULL; }
		$file->getDetails();
		return $file;
	}
	
	function commandLine($command) {
		return $this->mldonkey($command);
	}
	
/**
* Makes calls to mldonkey telnet service
* Pass in any number of commands; will return an array of
* lines corresponding to the command if one command is passed
* in, or an array of array of lines if more than one command
* was passed, corresponding to each command.  Each passed-in
* command can be an array if desired, and each element of the
* array will be treated as a command to run.
*
* @author   Moritz Warning <mwarning@users.sourceforge.net>
* @author   Thimo Langbehn <torian@users.sourceforge.net>
*/
	function mldonkey() {
		$retresults = array();
		$commands = array();
		$foundarray = FALSE;
		
		if(!$this->alive()) {
			return array();
		}
		
		$fp = fsockopen($this->host, $this->port, $errno, $errstr, $this->timeout);
		if(!$fp) {
			echo "Connection to host ".$this->host." port ".$this->port." cannot established!<br />\n";
			echo "ERROR ".$errno.": ".$errstr."<br />\n";
			exit();
		}
		
		//apply authentification
		if(!empty($this->user)) {
			$authstring = "auth ";
			if(!empty($this->user)) {
				$authstring .= $this->user." ";
			}
			$authstring .= ' "'.$this->passwd.'"';
			$commands = array($authstring);
		}
		
		// Force term_ansi to true - still needed?
		//$commands[] = "set term_ansi true";
	
		//get all commands into an array
		foreach(func_get_args() as $command) {
			if (is_array($command)) {
				$commands = array_merge($commands, $command);
				$foundarray = TRUE;
			} else {
				$commands[] = $command;
			}
		}

		$commandstring = implode("\n", $commands);
//echo "GMUI->MLD:<br>\n<pre>$commandstring</pre>";  //debug output
		fputs($fp, $commandstring."\n");
	
		$i = 0;
		$j = 0;
		$results = array();

		//finished contains the number of commandprompts left
		$finished = count($commands) + 1;
		while($finished > 0) {
		
                        if(feof($fp)) {
                                $finished = -1;
				break;
                        }
			
			$line = fgets($fp, 4096);
		       
			// Parse code lifted from phpeselgui
			// http://phpeselgui.sourceforge.net
			$line = preg_replace("/\[\dm/", "", $line);
			$line = preg_replace("/\[3\dm/", "", $line);
			$line = preg_replace("/\[2;37;0m/", "", $line);
			$line = preg_replace("/^\s*/", "", $line);
			$line = preg_replace("/\e/", "", $line);
			$line = preg_replace("/.*/", "", $line);
		
			 //catch exceptions
			//if(preg_match("/exception/", $results[$i])) {
			//	echo '<span class="error">'._("No valid output was received from MLDonkey!")."</span><br />\n";
			//	fclose($fp);
			//	return NULL;
			//}
			
			if(preg_match("/^MLdonkey command-line/", $line)) {
				--$finished;
			} else {
				$results[$i] = $line;
//echo "GMUI->MLD line ".$i.":<pre>".$results[$i]."</pre>";
				 if (trim($results[$i]) == trim("> > " . $commands[$j])) {
					reset($results);
					$retresults[] = $results;
					$results = array();
					$i = 0;
					$j++;
				} else {
					$i++;
					$delta = $finished + $j - count($commands) - 1;
					if($line != "") {
						if($delta != 0) {
							echo  '<span class="error">'._("Error during evaluation of parsed commands, MLD_Interface")."</span><br />\n";
						}
					}
				}
			}
		}
		
		if($finished == 0) {
			fputs($fp, "q\n");
		} else {
			echo '<span class="error">'._("Unexpected termination of connection, content may be missing")."</span><br />\n";
		}
		fclose($fp);

		$retresults[] = $results;
//echo "MLD->GMUI:<br>\n<pre>".print_r($retresults)."</pre>";  flush(); //debug output
		// Shift the Welcome message out
		array_shift($retresults);

		// Check for authorization
		if (preg_match("/^Command not authorized/", $retresults[0][0])) {
			exit('<span class="error">Error authenticating to MLDonkey.  Check your authentication settings in config.php</span><br>');
		} elseif (preg_match('/^Bad login\/password/', $retresults[0][0])) {
			exit('<span class="error">Bad username or password specified in config.php</span><br>');
		}
		// Shift authorization out
		if(!empty($this->user)) {
			array_shift($retresults);
		}
		
		// Shift our term_ansi check out
		//array_shift($retresults);
		
		// Check to see if term_ansi was false when we ran the commands
		if (count($retresults) == 0) {
			if (!in_array("kill\n", $commands) and !in_array("kill", $commands)) {
				$_SESSION['message'] .=  '<span class="error">'._("No valid output was received from MLDonkey!")."</span><br />\n";
				return array();
			}
		}
		
		// Now return everything
		if (func_num_args() > 1 OR $foundarray) {
			return $retresults;
		} elseif (isset($retresults[0])) {
			return $retresults[0];
		} else {
			return array();
		}
		
	}
}
?>